@@ -1,19 +1,20 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 import csv
 import sqlite3
 import argparse
 import sys,os
 import itertools
+from shutil import copyfile
 
-__version__ = '1.1.1'
+__version__ = '2.0.1'
 GEOIP_DB = os.path.join( os.path.dirname(__file__), 'geoip.db' )
 
 try:
 	db = sqlite3.connect(GEOIP_DB)
 except:
-	print "permission denied to open %s" % GEOIP_DB
+	print("permission denied to open %s" % GEOIP_DB)
 	exit()
-	
+
 db.text_factory = str
 sql = db.cursor()
 
@@ -22,7 +23,7 @@ squares = []
 circles = []
 
 arg_parser = argparse.ArgumentParser()
-arg_parser.add_argument("-update", dest='update', action="store_true", help='update local database from remote ZIP-archive')
+arg_parser.add_argument("-update", dest='update', nargs='?', const='', help='update local database from remote ZIP-archive')
 arg_parser.add_argument("-info", dest='info', action="store_true", help='show total amount netblocks')
 
 arg_parser.add_argument('-ip', dest="ipaddr", action="append", help='search network by IP')
@@ -39,7 +40,7 @@ arg_parser.add_argument('-circle', dest="lat_long_km", action="append", help='se
 arg_parser.add_argument("-resolve-rwhois", dest="resolve_ripe", action="store_true", help="rwhois db resolve netname (faster)")
 arg_parser.add_argument("-resolve-whois", dest="resolve_whois", action="store_true", help="whois resolve netname (slower)")
 
-arg_parser.add_argument("-kml", dest="save_as_kml", action="store_true", help="save coordinates of netblocks as KML")
+arg_parser.add_argument("-kml", dest="save_to_kml", help="save coordinates of netblocks as KML")
 arg_parser.add_argument("-html", dest="save_to_html", help="save coordinates of netblocks as HTML")
 arg_parser.add_argument("-version", dest="version", action="store_true", help="show version")
 
@@ -54,7 +55,7 @@ def check_db():
 
 def show_db_info():
 	count, = sql.execute('SELECT COUNT(network) FROM geoip')
-	print count[0]
+	print(count[0])
 
 def cidr_to_min_max(cidr):
 	if len( cidr.split('/') ) == 2:
@@ -68,19 +69,19 @@ def cidr_to_min_max(cidr):
 	_max = _min + mask
 	return _min,_max
 
-def update(tmpfile):
-	import urllib2
+def update(tmpfile, url=None):
+	import urllib.request
 	from zipfile import ZipFile
-	from io import BytesIO
+	from io import StringIO
 
 	DB_ASN = "http://web.archive.org/web/20191227183143/https://geolite.maxmind.com/download/geoip/database/GeoLite2-ASN-CSV.zip"
 	DB_COUNTRY = "http://web.archive.org/web/20191227183011/https://geolite.maxmind.com/download/geoip/database/GeoLite2-Country-CSV.zip"
 	DB_CITY = "http://web.archive.org/web/20191227182816if_/https://geolite.maxmind.com/download/geoip/database/GeoLite2-City-CSV.zip"
 
 	def download(uri,target):
-		print uri
-		resp = urllib2.urlopen(uri)
-		size = int( resp.headers.getheader('content-length') or 0 )
+		print(uri)
+		resp = urllib.request.urlopen(uri)
+		size = int( resp.headers.get('content-length') or resp.headers.get('x-archive-orig-content-length') or 0 )
 		downloaded = 0
 		while True:
 			data = resp.read(4096)
@@ -95,41 +96,44 @@ def update(tmpfile):
 				sys.stdout.write( "\r%d bytes" % downloaded )
 			sys.stdout.flush()
 
-	
-	download(uri=DB_CITY, target=tmpfile)
-	print '\nunpacking...'
+	if url and os.path.isfile(url):
+		copyfile(url, tmpfile.name)
+	else:
+		download(uri=url or DB_CITY, target=tmpfile)
+	print( '\nunpacking...' )
 	z = ZipFile( tmpfile )
-	lang = raw_input("select language (ja/zh-CN/fr/ru/en/pt-BR/de/es): ")
+	lang = input("select language (ja/zh-CN/fr/ru/en/pt-BR/de/es): ")
 	db_blocks = ''
 	db_locations = ''
 	for compressed_filepath in z.namelist():
 		if compressed_filepath.find('GeoLite2-City-Blocks-IPv4.csv') != -1:
-			db_blocks = z.read(compressed_filepath)
+			db_blocks = z.read(compressed_filepath).decode()
 		elif compressed_filepath.find('GeoLite2-City-Locations-%s.csv' % lang) != -1:
-			db_locations = z.read(compressed_filepath)
+			db_locations = z.read(compressed_filepath).decode()
 	if not db_blocks:
-		print "'GeoLite2-City-Blocks-IPv4.csv' not found in %s" % DB_CITY
+		print( "'GeoLite2-City-Blocks-IPv4.csv' not found in %s" % DB_CITY )
 		return False
 	elif not db_locations:
-		print "'GeoLite2-City-Locations-%s.csv' not found in %s" % (lang,DB_CITY)
+		print( "'GeoLite2-City-Locations-%s.csv' not found in %s" % (lang,DB_CITY) )
 		return False
 	z.close()
 
-	print 'importing...'
+	print( 'importing...' )
 	sql.execute("DROP TABLE IF EXISTS geoip")
 	sql.execute("CREATE TABLE geoip(ip_begin INT, ip_end INT, network TEXT, asn TEXT, org TEXT, continent TEXT, country TEXT, city TEXT, lat FLOAT, long FLOAT)")
 	locations = {}
-	for location in csv.DictReader( BytesIO(db_locations) ):
+	
+	for location in csv.DictReader(StringIO(db_locations)):
 		locations.update( { 
 			location['geoname_id'] : {
-				'continent': location['continent_name'].decode('utf-8').lower(),
-				'country': location['country_name'].decode('utf-8').lower(),
-				'city': location['city_name'].decode('utf-8').lower()
+				'continent': location['continent_name'].lower(),
+				'country': location['country_name'].lower(),
+				'city': location['city_name'].lower()
 			}
 		} )
 
 	n = 0
-	for block in csv.DictReader( BytesIO(db_blocks) ):
+	for block in csv.DictReader(StringIO(db_blocks)):
 		location = locations.get( block['geoname_id'] )
 		if location:
 			continent = location['continent']
@@ -154,28 +158,31 @@ def update(tmpfile):
 	sys.stdout.write("\r%d networks\n" % n)
 	sys.stdout.flush()
 
-	tmpfile.truncate()
-	download(uri=DB_ASN, target=tmpfile)
-	print '\nunpacking...'
-	z = ZipFile( tmpfile )
-	db_asn = ''
-	for compressed_filepath in z.namelist():
-		if compressed_filepath.find('GeoLite2-ASN-Blocks-IPv4.csv') != -1:
-			db_asn = z.read(compressed_filepath)
-	if not db_asn:
-		print "'GeoLite2-ASN-Blocks-IPv4.csv' not found in %s" % DB_ASN
-		return False
-	print 'importing...'
-	n = 1
-	for asn in csv.DictReader( BytesIO(db_asn) ):
-		net = asn['network']
-		num = asn['autonomous_system_number']
-		org = asn['autonomous_system_organization']
-		sql.execute( "UPDATE geoip SET asn=?, org=? WHERE network = ?", (num,org,net) )
-		if n % 25000 == 0:
-			sys.stdout.write("\r%d ASNs" % n)
-			sys.stdout.flush()
-		n += 1
+	try:
+		tmpfile.truncate()
+		download(uri=DB_ASN, target=tmpfile)
+		print( '\nunpacking...' )
+		z = ZipFile( tmpfile )
+		db_asn = ''
+		for compressed_filepath in z.namelist():
+			if compressed_filepath.find('GeoLite2-ASN-Blocks-IPv4.csv') != -1:
+				db_asn = z.read(compressed_filepath)
+		if not db_asn:
+			print( "'GeoLite2-ASN-Blocks-IPv4.csv' not found in %s" % DB_ASN )
+			return False
+		print( 'importing...' )
+		n = 1
+		for asn in csv.DictReader( BytesIO(db_asn) ):
+			net = asn['network']
+			num = asn['autonomous_system_number']
+			org = asn['autonomous_system_organization']
+			sql.execute( "UPDATE geoip SET asn=?, org=? WHERE network = ?", (num,org,net) )
+			if n % 25000 == 0:
+				sys.stdout.write("\r%d ASNs" % n)
+				sys.stdout.flush()
+			n += 1
+	except Exception as e:
+		print(str(e))
 
 	db.commit()
 	sys.stdout.write("\r%d ASNs\n" % n)
@@ -188,7 +195,7 @@ def do_search(items, params):
 	args = []
 	def _sign(l):
 		l = l.upper()
-		for piece,sign in {'N':1, 'S':-1, 'E': 1, 'W': -1}.items():
+		for piece,sign in list({'N':1, 'S':-1, 'E': 1, 'W': -1}.items()):
 			if l.find(piece) != -1:
 				return float( l.replace(piece,'') ) * sign
 		return float(l)
@@ -199,15 +206,15 @@ def do_search(items, params):
 			from_longitude,to_longitude = to_longitude,from_longitude
 		return from_latitude,from_longitude,to_latitude,to_longitude
 
-	for attr,val in params.items():
+	for attr,val in list(params.items()):
 		if attr == 'square':
-			(from_latitude,from_longitude,to_latitude,to_longitude) = _check_square( *map( _sign, val.split(',') ) )
+			(from_latitude,from_longitude,to_latitude,to_longitude) = _check_square( *list(map( _sign, val.split(',') ) ))
 			statement.append( "(lat >= ? AND lat <= ? AND long >= ? AND long <= ?)" )
 			args.extend( [from_latitude, to_latitude, from_longitude, to_longitude] )
 			squares.append( [from_latitude, from_longitude, to_latitude, to_longitude] )
 		elif attr == 'circle':
 			lat, lon, radius_km = val.split(',')
-			lat, lon = map( _sign, [lat, lon] )
+			lat, lon = list(map( _sign, [lat, lon] ))
 			radius = float(radius_km) * ( 1.0 / 110.574 )
 			#radius = float(radius_km) * ( 1.0 / ( 111.320*math.cos(lat) ) )
 			statement.append( "( ( (lat - ?)*(lat - ?) + (long - ?)*(long - ?) ) < ? )" )
@@ -234,21 +241,19 @@ def do_search(items, params):
 
 	results = []
 	query = ( "SELECT %s FROM geoip WHERE " % ','.join(items) ) + ' AND '.join(statement)
-	#print query
-	#print args
 	for result in sql.execute( query, args ):
-		results.append( dict( zip(items,result) ) )
+		results.append( dict( list(zip(items,result)) ) )
 	return results
 
 def search(items, params):
 	results = []
-	for attrs in itertools.product( *params.values() ):
-		results += do_search( items, dict( zip(params.keys(), attrs) ) )
+	for attrs in itertools.product( *list(params.values()) ):
+		results += do_search( items, dict( list(zip(list(params.keys()), attrs)) ) )
 	return results
 
 def geo_search(args):
 	params = {}
-	for attr,vals in args.items():
+	for attr,vals in list(args.items()):
 		params[attr] = []
 		for val in vals:
 			if os.path.isfile(val):
@@ -260,7 +265,7 @@ def geo_search(args):
 			elif val == '-':
 				while True:
 					try:
-						val = raw_input()
+						val = input()
 						params[attr].append(val)
 					except:
 						break
@@ -287,7 +292,7 @@ def resolve_ripe(netblocks):
 			if netname:
 				netblock['netname'] = netname[:20]+'..' if len(netname) > 20 else netname
 
-def to_kml(netblocks):
+def save_kml(netblocks, outfile):
 	from pykml.factory import KML_ElementMaker as KML
 	from lxml import etree
 	import math
@@ -333,7 +338,7 @@ def to_kml(netblocks):
 			KML.LineString(
 				KML.coordinates(
 					' '.join( map( lambda xy:'%.04f,%.04f,0.0 '%(xy[0],xy[1]), 
-							[ ( longitude+math.cos(2*math.pi/n*x)*radius, latitude+math.sin(2*math.pi/n*x)*radius ) for x in xrange(0,n+1) ]
+							[ ( longitude+math.cos(2*math.pi/n*x)*radius, latitude+math.sin(2*math.pi/n*x)*radius ) for x in range(0,n+1) ]
 						)
 					)
 				)
@@ -350,14 +355,16 @@ def to_kml(netblocks):
 				points[point].append( ' '.join( [network,netname] ) )
 			else:
 				points[point] = [network]
-	for point in points.keys():
-		lat,lon = map( float, point.split("/") )
+	for point in list(points.keys()):
+		lat,lon = list(map( float, point.split("/") ))
 		places.append( kml( "\n".join( points[point] ), lat, lon ) )
 	for square in squares:
-		places.append( draw_square( *map( float, square ) ) )
+		places.append( draw_square( *list(map( float, square ) ) ))
 	for circle in circles:
-		places.append( draw_circle( *map( float, circle[:3] ) ) )
-	return etree.tostring( KML.Folder( *tuple(places) ) )
+		places.append( draw_circle( *list(map( float, circle[:3] ) ) ))
+	
+	with open(outfile, "wb") as o:
+		o.write( etree.tostring( KML.Folder( *tuple(places) ) ) )
 
 def html_escape(text):
 	return text.replace("`", "'").replace("'", "&#x27;").replace('"', "&quot;")
@@ -368,22 +375,22 @@ def save_html(netblocks, items, outfile):
 
 	coordinates = {}
 	for netblock in netblocks:
-		about_netblock = ' | '.join( map( lambda i: str( netblock.get(i) or '' ).decode('utf-8'), items ) )
+		about_netblock = ' | '.join( [str( netblock.get(i) or '' ) for i in items] )
 		if netblock.get('lat') and netblock.get('long'):
-			if "%.04f,%.04f" % ( netblock.get('lat'), netblock.get('long') ) in coordinates.keys():
+			if "%.04f,%.04f" % ( netblock.get('lat'), netblock.get('long') ) in list(coordinates.keys()):
 				coordinates[ "%.04f,%.04f" % ( netblock.get('lat'), netblock.get('long') ) ] += "<br>" + html_escape(about_netblock)
 			else:
 				coordinates[ "%.04f,%.04f" % ( netblock.get('lat'), netblock.get('long') ) ] = html_escape(about_netblock)
 		
-	for lat_long,networks in coordinates.items():
-		folium.CircleMarker(location=map(float, lat_long.split(',')), popup=networks, radius=1).add_to(folium_map)
+	for lat_long,networks in list(coordinates.items()):
+		folium.CircleMarker(location=list(map(float, lat_long.split(','))), popup=networks, radius=1).add_to(folium_map)
 
 	for circle in circles:
-		folium.Circle(location=map(float, circle[:2]), radius=circle[3]*1000, color="red").add_to(folium_map)
+		folium.Circle(location=list(map(float, circle[:2])), radius=circle[3]*1000, color="red").add_to(folium_map)
 
 	for square in squares:
-		print map(float, square)
-		from_latitude, from_longitude, to_latitude, to_longitude = map(float, square)
+		print( list(map(float, square)) )
+		from_latitude, from_longitude, to_latitude, to_longitude = list(map(float, square))
 		locations = [ (from_latitude,from_longitude) ]
 		locations.append( (from_latitude,to_longitude) )
 		locations.append( (to_latitude,to_longitude) )
@@ -399,22 +406,22 @@ def get_stat(netblocks, items):
 	ips = 0
 	for item in items:
 		if item == 'network':
-			for network in map( lambda n: n.get('network'), netblocks ):
+			for network in [n.get('network') for n in netblocks]:
 				_min,_max = cidr_to_min_max(network)
 				ips += _max - _min
 			statistics[item] = '%d ip' % ips
 		else:
 			vals = set()
-			for val in map( lambda n: str(n.get(item)) or '', netblocks ):
+			for val in [str(n.get(item)) or '' for n in netblocks]:
 				vals.add(val)
 			statistics[item] = '%d %s' % ( len(vals), item )
 	return statistics
 
 def print_row( values, margins ):
 	row = []
-	for i in xrange( len(values) ):
-		row.append( values[i] + " " * ( margins[i] - len( values[i].decode('utf-8') ) ) )
-	print ' | '.join(row)
+	for i in range( len(values) ):
+		row.append( values[i] + " " * ( margins[i] - len( values[i]) ) )
+	print(' | '.join(row))
 
 
 def main( argv=['-h'] ):
@@ -425,14 +432,17 @@ def main( argv=['-h'] ):
 	netblocks = []
 
 	if args.version:
-		print __version__
-	elif args.update:
+		print(__version__)
+	elif args.update != None:
 		from tempfile import NamedTemporaryFile
 		tmpfile = NamedTemporaryFile()
 		try:
-			update(tmpfile)
+			if args.update:
+				update(tmpfile, url=args.update)
+			else:
+				update(tmpfile)
 		except Exception as e:
-			print str(e)
+			print(str(e))
 		tmpfile.close()
 	elif args.info:
 		show_db_info()
@@ -469,7 +479,7 @@ def main( argv=['-h'] ):
 			if check_db():
 				netblocks = geo_search( params )
 			else:
-				print "update database first"
+				print( "update database first" )
 				return
 
 	if args.resolve_ripe:
@@ -479,28 +489,29 @@ def main( argv=['-h'] ):
 		resolve_whois( netblocks )
 		items.insert(1, "netname")
 
-	if netblocks and args.save_as_kml:
-		print to_kml( netblocks )
+	if netblocks and args.save_to_kml:
+		save_kml(netblocks, args.save_to_kml)
 	elif netblocks and args.save_to_html:
 		items.remove('lat')
 		items.remove('long')
-		save_html( netblocks, items, args.save_to_html )
+		save_html(netblocks, items, args.save_to_html)
 	elif netblocks:
 		summary = get_stat(netblocks, items)
-		margins = map( lambda i: max( map( lambda n: len(str(n.get(i) or '').decode('utf-8')), netblocks ) + [len(i), len(summary[i])] ), items )
+		margins = [max( [len(str(n.get(i) or '')) for n in netblocks] + [len(i), len(summary[i])] ) for i in items]
 		if len(items) > 1:
-			print_row( tuple(items), margins )
-			print_row( tuple( map( lambda m: '-'*m, margins ) ), margins )
+			print_row(tuple(items), margins )
+			print_row(tuple( ['-'*m for m in margins] ), margins )
 			for netblock in netblocks:
-				print_row( map( lambda i: str( netblock.get(i) or '' ), items ), margins )
-			print_row( tuple( map( lambda m: '-'*m, margins ) ), margins )
-			print_row( tuple( map( lambda i: str( summary.get(i) or '' ), items ) ), margins )
+				print_row([str( netblock.get(i) or '' ) for i in items], margins )
+			print_row(tuple( ['-'*m for m in margins] ), margins )
+			print_row(tuple( [str( summary.get(i) or '' ) for i in items] ), margins )
+
 		else:
 			for netblock in netblocks:
-				print_row( map( lambda i: str( netblock.get(i) or '' ), items ), [0] )
+				print_row(([str( netblock.get(i) or '' ) for i in items], [0] ))
 
 	if db:
 		db.close()
 
 if __name__ == '__main__':
-	main( sys.argv[1:] )
\ No newline at end of file
+	main( sys.argv[1:] )
